{% extends 'layout.html'%}

{%block content%}
<br>
<main>
    <a href = "/">Home page</a>
    <br>
    <a href = "/device_codes">Device Codes</a>
    <br>
    <a href = "/refresh_tokens">Refresh Tokens</a>
    <br>
    <a href = "/access_tokens">Access Tokens</a>
    <br>
    <a href = "/graph_requests">Graph Requests</a>
    <br>
    <a href = "/generic_search">Generic Search</a>
    <br>
    <a href = "/recent_files">Recent Files</a>
    <br>
    <a href = "/shared_with_me">Files Shared With Me</a>
    <br>
    <a href = "/onedrive">OneDrive</a>
    <br>
    <a href = "/sharepoint_sites">SharePoint Sites</a>
    <br>
    <a href = "/sharepoint_drives">SharePoint Drives</a>
    <br>
    <a href = "/sharepoint">SharePoint Files</a>
    <br>
    <a href = "/outlook">Outlook</a>
</main>
{%endblock content%}